@@ -1,34 +1,32 @@
-/* $ID: power_method.c, last updated 2020-08-21, F.Osorio */
+/* $ID: power_method.c, last updated 2020-09-04, F.Osorio */
 
-#include "base.h"
-#include "power_method.h"
-#include "vector.h"
+#include "fastmatrix.h"
 
 void
 power_method(double *a, int *lda, int *nrow, int *ncol, double *x, double *lambda,
   int *maxiter, double *tolerance, int *numIter)
 { /* power method to approximate dominant eigenvalue and eigenvector */
   char *notrans = "N";
-  double conv, div, newLambda, one = 1.0, zero = 0.0, *z = NULL;
-  int inc = 1, iter = 0, n = *nrow, p = *ncol;
+  double conv, div, newLambda, *z = NULL;
+  int iter = 0, n = *nrow, p = *ncol;
 
   z = (double *) Calloc(n, double);
 
   /* normalizing initial vector */
-  normalize_vec(x, inc, n);
+  normalize_vec(x, 1, n);
 
   /* initial estimate */
-  F77_CALL(dgemv)(notrans, &n, &p, &one, a, lda, x, &inc, &zero, z, &inc);
-  normalize_vec(z, inc, n);
-  *lambda = F77_CALL(ddot)(&n, x, &inc, z, &inc);
+  BLAS2_gemv(1.0, a, *lda, n, p, notrans, x, 1, 0.0, z, 1);
+  normalize_vec(z, 1, n);
+  *lambda = BLAS1_dot_product(x, 1, z, 1, n);
   Memcpy(x, z, p); /* x <- z */
 
   /* main loop */
   repeat {
-    F77_CALL(dgemv)(notrans, &n, &p, &one, a, lda, x, &inc, &zero, z, &inc);
-    normalize_vec(z, inc, n);
-    F77_CALL(dgemv)(notrans, &n, &p, &one, a, lda, z, &inc, &zero, x, &inc);
-    newLambda = F77_CALL(ddot)(&n, x, &inc, z, &inc);
+    BLAS2_gemv(1.0, a, *lda, n, p, notrans, x, 1, 0.0, z, 1);
+    normalize_vec(z, 1, n);
+    BLAS2_gemv(1.0, a, *lda, n, p, notrans, z, 1, 0.0, x, 1);
+    newLambda = BLAS1_dot_product(x, 1, z, 1, n);
 
     iter++;
 
@@ -44,7 +42,7 @@ power_method(double *a, int *lda, int *nrow, int *ncol, double *x, double *lambd
   }
   *lambda = newLambda;
   div = 1.0 / *lambda;
-  F77_CALL(dscal)(&n, &div, x, &inc);
+  BLAS1_scale(div, x, 1, n);
 
   *numIter = iter;
 
