@@ -25,6 +25,26 @@ normalize_vec(double *x, int inc, int n)
 
 /* matrix operations */
 
+void
+FM_cov2cor(double *cov, int p)
+{ /* scales a 'covariance' matrix into the corresponding correlation matrix */
+  double *s;
+
+  s = (double *) Calloc(p, double);
+
+  for (int i = 0; i < p; i++)
+    s[i] = cov[i * (p + 1)];
+
+  for (int i = 0; i < p; i++) {
+    cov[i * (p + 1)] = 1.0;
+    for (int j = i + 1; j < p; j++) {
+      *(cov + i + j * p) = *(cov + i + j * p) / sqrt(s[i] * s[j]);
+      *(cov + j + i * p) = *(cov + i + j * p);
+    }
+  }
+  Free(s);
+}
+
 void
 mat2vech(double *x, int *ldx, int *n, double *y)
 { /* y <- vech(x) */
